package de.fraunhofer.iem.dataprovider.taskManager

import de.fraunhofer.iem.dataprovider.logger.getLogger
import jakarta.annotation.PreDestroy
import kotlinx.coroutines.delay
import org.springframework.stereotype.Component

enum class TaskType {
    REPO_CHANGED,
    DONE
}
interface Task {
    val type: TaskType
    suspend fun execute(): Unit
}

class GitTask(
    override val type: TaskType,
    private val gitUrl: String,
    private val responseChannel: suspend (task: Task) -> Unit
): Task {
    override suspend fun execute() {
        println("Cloning $gitUrl")
        delay(5000)
        responseChannel(DoneTask("Done 1 with gitUrl $gitUrl"))
        delay(5000)
        responseChannel(DoneTask("Done 2 with gitUrl $gitUrl"))
        //        val git: Git = Git.cloneRepository()
//            .setURI("https://github.com/eclipse/jgit.git")
//            .setDirectory("/path/to/repo")
//            .call()
    }
}

class DoneTask(private val message: String? = null): Task {
    override val type: TaskType = TaskType.DONE
    override suspend fun execute() {
        println("[${Thread.currentThread().name}] I'm done $message")
    }
}


/**
 * The Task Manager takes tasks and distributes them to
 * underlying workers. Internally it uses a channel
 * to manage incoming tasks.
 */
@Component
class TaskManager: IWorker {

    private val worker = Worker("Worker")
    private val logger = getLogger(javaClass)

    override suspend fun addTask(task: Task) {
        if (task.type == TaskType.DONE) {
            task.execute()
        }
        logger.info("[${Thread.currentThread().name}] add task called in Task Manager $task")
        worker.addTask(task)
        logger.info("[${Thread.currentThread().name}] add task finished in Task Manager $task")
    }

    @PreDestroy
    override fun close() {
        worker.close()
    }
}


